#! /usr/bin/perl
#
# Script: perl RACalculator.pl
# Description: 1) Count the number of times each peptide appears in a protein;
#              2) Each protein associated with all the peptides and occurrence;
#              3) All peptide sequences with their fraction numbers in the sample;
#              4) Assign unique identifier to each peptide sequence;
#              5) Each protein associated with all the peptide ids and occurrence;
#              6) Each peptide associated with all the proteins which contain it;
#              7) Identify shared and unique peptides;
#			         8) Group proteins based on shared peptides;
#			         9) Based on step 7 and 8, determine spectral counts for shared and unique peptides;
#			         10) Calculate adjusted spectral counts of peptides based on the formula.
#              11) Normalize adjusted spectral counts
#              12) Scale adjusted spectral counts
#
# Input file: AGIList.txt, Bio3_combined_F16-49_fillin.reduced.txt, protein_length.txt
#
# Output file: scaling.txt
#
# Author: Yi Xiong, Qing Xu
# Email: xiongyi128@gmail.com, xu276@purdue.edu
# Date: Sept. 18th, 2014

###########################################################################
#The AGIList.txt file contains all the protein IDs in the given sample
###########################################################################
#print "start\n";
my $infile = "AGIList.txt";
open( IN, $infile ) or die "open fail";
@idarray = <IN>;
close(IN);
use Data::Dumper;
#print Dumper(@idarray);

###########################################################################
#Assign the IDs to the hash
###########################################################################
my %idhash;
foreach my $i ( 0 .. $#idarray ) {

	$idarray[$i] =~ s/\A\s+|\s+\z//g;
	$idhash{ $idarray[$i] } = 1;
}
#print Dumper(%idhash);

###########################################################################
#Peptides assigned to proteins by Mascot
###########################################################################
$infile = "Bio3_combined_F16-49_fillin.reduced.txt";
open( IN, $infile ) or die "open fail";
@segmentarray = <IN>;
close(IN);

my %allpeptidehash;
my %allpeptideseqhash;

###########################################################################
#Count the number of times each peptide appears in a protein
###########################################################################
$outfile = "Bio3_combined_F16-49_fillin.reduced.count.txt";

###########################################################################
#Each protein associated with all the peptides and occurrence
###########################################################################
$outppfile = "Bio3_protein.peptide.count.txt";
open( OUT,   ">$outfile" );
open( OUTPP, ">$outppfile" );

foreach my $key ( sort keys %idhash ) {

	my %peptidehash;
	foreach my $i ( 0 .. $#segmentarray ) {

		$segmentarray[$i] =~ s/\A\s+|\s+\z//g;
		@tmparray = split( /\s+/, $segmentarray[$i] );
		if ( $key eq $tmparray[0] ) {
			$peptide = $tmparray[1] . "\t" . $tmparray[2];
			$allpeptidehash{$peptide} = 1;
			$allpeptideseqhash{ $tmparray[1] } = 1;
			if ( !exists $peptidehash{$peptide} ) {
				$peptidehash{$peptide} = 1;
			}
			else {
				$peptidehash{$peptide} = $peptidehash{$peptide} + 1;
			}
		}
	}
    
	print OUTPP $key . "\t";

	foreach my $tmpkey ( sort keys %peptidehash ) {
		print OUT $key . "\t" . $tmpkey . "\t" . $peptidehash{$tmpkey} . "\n";
		@tmparray = split( /\s+/, $tmpkey );
		print OUTPP $tmparray[0] . "-"
		  . $tmparray[1] . "-"
		  . $peptidehash{$tmpkey} . "\t";
	}

	print OUTPP "\n";
}
close(OUT);
close(OUTPP);

###########################################################################
# All peptide sequences with their fraction numbers in the sample
###########################################################################
$outfile = "Bio3_all.peptide.txt";
open( OUT, ">$outfile" );
foreach my $key ( sort keys %allpeptidehash ) {

	print OUT $key . "\n";
}
close(OUT);

my $id = -1;
foreach my $key ( sort keys %allpeptideseqhash ) {

	++$id;
	if ( $id < 10 ) {
		$seqid = "000" . $id;
	}
	elsif ( $id < 100 ) {
		$seqid = "00" . $id;
	}
	elsif ( $id < 1000 ) {
		$seqid = "0" . $id;
	}
	else {
		$seqid = $id;
	}
	$allpeptideseqhash{$key} = $seqid;
}

###########################################################################
#Assign unique identifier to each peptide sequence
###########################################################################
$outfile = "Bio3_all.peptide.mapping.txt";
open( OUT, ">$outfile" );
foreach my $key ( sort keys %allpeptideseqhash ) {

	print OUT $key . "\t" . $allpeptideseqhash{$key} . "\n";
}
close(OUT);

###########################################################################
#Each protein associated with all the peptide ids and occurrence
###########################################################################
$infile = "Bio3_all.peptide.mapping.txt";
open( IN, $infile );
@idarray = <IN>;
close(IN);

my %allpeptideseqhash;
foreach my $i ( 0 .. $#idarray ) {

	@tmparray = split( /\s+/, $idarray[$i] );
	$allpeptideseqhash{ $tmparray[0] } = $tmparray[1];
}

$infile = "Bio3_protein.peptide.count.txt";
open( IN, $infile ) or die "open fail";
@segmentarray = <IN>;
close(IN);

$outfile = "Bio3_protein.peptide.count.mapping.txt";
open( OUT, ">$outfile" );

foreach my $i ( 0 .. $#segmentarray ) {

	@tmparray = split( /\s+/, $segmentarray[$i] );
	print OUT $tmparray[0] . "\t";
	foreach my $j ( 1 .. $#tmparray ) {
		@idarray = split( /\-/, $tmparray[$j] );
		print OUT $allpeptideseqhash{ $idarray[0] } . "-"
		  . $idarray[1] . "-"
		  . $idarray[2] . "\t";
	}
	print OUT "\n";
}
close(OUT);

##################################################################################
#Each protein associated with all the peptide ids and occurrence in each fraction
##################################################################################
$infile = "Bio3_protein.peptide.count.mapping.txt";
open( IN, $infile );
@protein_pepidlist_array = <IN>;
close(IN);

foreach my $id ( 16 .. 49 ) {

	$fid = "F" . $id;

	$outfile = "Bio3_protein.peptide.count.mapping_$fid.txt";
	open( OUT, ">$outfile" );

	foreach my $i ( 0 .. $#protein_pepidlist_array ) {
		my @tmparray = split( /\s+/, $protein_pepidlist_array[$i] );

		my $pepidlist_str = "";
		foreach my $j ( 1 .. $#tmparray ) {
			if ( substr( $tmparray[$j], 5, 3 ) eq $fid ) {
				$pepidlist_str .= $tmparray[$j] . "\t";
			}
		}

		if ( length($pepidlist_str) > 0 ) {
			print OUT $tmparray[0] . "\t" . $pepidlist_str . "\n";
		}
	}
}
close(OUT);

###
$infile = "AGIList.txt";
open( IN, $infile );
@ppidarray = <IN>;
close(IN);
chomp(@ppidarray);









foreach my $id ( 16 .. 49 ) {

	$fid = "F" . $id;
	$infile = "Bio3_protein.peptide.count.mapping_$fid.txt";
	open( IN, $infile ) or die "open Bio3_protein.peptide.count.mapping_$fid.txt fail ";
	@protein_pepidlist_array = <IN>;
	close(IN);

	my %protein_pepidlist_hash;
	my @pidarray;
	foreach my $i ( 0 .. $#protein_pepidlist_array ) {
		my @tmparray = split( /\s+/, $protein_pepidlist_array[$i] );
		$pepidlist_str = "";
		foreach my $j ( 1 .. $#tmparray ) {
			$pepidlist_str .= $tmparray[$j] . "\t";
		}

		# AT1G80600.1	0648-F29-1	0694-F29-1	1141-F27-2
		# the first is the key, the rest peptide id list is the value
		$protein_pepidlist_hash{ $tmparray[0] } = $pepidlist_str;
		push @pidarray, $tmparray[0];
	}

	$outfile = "AGIList_abundance_$fid.txt";
	open( OUT, ">$outfile" );

	my %protein_with_share_pep_list_hash;
	my %protein_with_unique_pep_list_hash;

	my %share_pep_to_protein_list_hash;

	foreach my $i ( 0 .. $#pidarray ) {    #

		$this_pid = $pidarray[$i];

		my %this_protein_pepidlist_hash;
		my @tmparray = split( /\s+/, $protein_pepidlist_hash{$this_pid} );
		foreach my $k ( 0 .. $#tmparray ) {
			$this_protein_pepidlist_hash{ substr( $tmparray[$k], 0, 8 ) } = 1;
		}

		my %other_protein_pepidlist_hash;
		foreach my $j ( 0 .. $#pidarray ) {
			if ( $j != $i ) {
				my @tmparray =
				  split( /\s+/, $protein_pepidlist_hash{ $pidarray[$j] } );
				foreach my $k ( 0 .. $#tmparray ) {
					$other_protein_pepidlist_hash{ substr( $tmparray[$k], 0, 8 )
					  } = 1;
				}
			}
		}

		my $tmp_unique_str = "";
		my $tmp_share_str  = "";

		foreach my $key ( sort keys %this_protein_pepidlist_hash ) {
			if ( !exists $other_protein_pepidlist_hash{$key} ) {
				$tmp_unique_str .= $key . "\t";
			}
			else {
				$tmp_share_str .= $key . "\t";
			}
		}

		if ( $tmp_unique_str ne "" ) {
			$protein_with_unique_pep_list_hash{$this_pid} = $tmp_unique_str;
		}
		if ( $tmp_share_str ne "" ) {
			$protein_with_share_pep_list_hash{$this_pid} = $tmp_share_str;
		}
	}

	###########################################################################
	# Each peptide associated with all the proteins which contain it
	###########################################################################
	foreach my $key ( sort keys %protein_with_share_pep_list_hash ) {

		my @tmparray = split( /\s+/, $protein_with_share_pep_list_hash{$key} );
		foreach my $element (@tmparray) {
			if ( !exists $share_pep_to_protein_list_hash{$element} ) {
				$share_pep_to_protein_list_hash{$element} = $key;
			}
			else {
				$share_pep_to_protein_list_hash{$element} =
				  $share_pep_to_protein_list_hash{$element} . "\t" . $key;
			}
		}
	}

	###########################################################################
	# Each group of proteins is identified by a common list of peptides
	###########################################################################
	my %protein_group_as_key_peptide_list_hash;
	foreach my $key ( sort keys %share_pep_to_protein_list_hash ) {

		my @tmparray = split( /\s+/, $share_pep_to_protein_list_hash{$key} );
		my $protein_group = "";
		foreach my $i ( 0 .. $#tmparray ) {
			$protein_group .= $tmparray[$i] . "\t";
		}
		if ( !exists $protein_group_as_key_peptide_list_hash{$protein_group} ) {
			$protein_group_as_key_peptide_list_hash{$protein_group} = $key;
		}
		else {
			$protein_group_as_key_peptide_list_hash{$protein_group} =
			    $protein_group_as_key_peptide_list_hash{$protein_group} . "\t"
			  . $key;
		}
	}

	my %unique_spectral_count_hash;
	
	
	###########################################################################
	#Unique spectral count separate/sum
	###########################################################################
	my %uSPC_hash;
	foreach my $key ( sort keys %protein_with_unique_pep_list_hash ) {

		my %peptide_num_ofthisprotein_hash;

		my @tmparray = split( /\s+/, $protein_pepidlist_hash{$key} );
		foreach my $element (@tmparray) {
			my $peptide_id = substr( $element, 0, 8 );
			my @elementarray = split( /\-/, $element );
			$peptide_num_ofthisprotein_hash{$peptide_id} = $elementarray[2];
		}

		my $peptide_num_sum = 0;
		@tmparray =
		  split( /\s+/, $protein_with_unique_pep_list_hash{$key} );
		foreach my $element (@tmparray) {
			if ( exists $peptide_num_ofthisprotein_hash{$element} ) {
				$peptide_num_sum += $peptide_num_ofthisprotein_hash{$element};
			}
		}

		$uSPC_hash{$key} = $peptide_num_sum;
		my $num_of_peptide_type = $#tmparray + 1;

		$unique_spectral_count_hash{$key} =
		  $peptide_num_sum . "(" . $num_of_peptide_type . ")";
	}

	my %share_spectral_count_hash;

	###########################################################################
	#Share spectral count separate/sum
	###########################################################################
	foreach my $key ( sort keys %protein_with_share_pep_list_hash ) {

		my %peptide_num_ofthisprotein_hash;

		my @tmparray = split( /\s+/, $protein_pepidlist_hash{$key} );
		foreach my $element (@tmparray) {
			my $peptide_id = substr( $element, 0, 8 );
			my @elementarray = split( /\-/, $element );
			$peptide_num_ofthisprotein_hash{$peptide_id} = $elementarray[2];
		}

		my $peptide_num_sum = 0;
		@tmparray =
		  split( /\s+/, $protein_with_share_pep_list_hash{$key} );
		foreach my $element (@tmparray) {
			if ( exists $peptide_num_ofthisprotein_hash{$element} ) {
				$peptide_num_sum += $peptide_num_ofthisprotein_hash{$element};
			}
		}

		my $num_of_peptide_type = $#tmparray + 1;

		$share_spectral_count_hash{$key} =
		  $peptide_num_sum . "(" . $num_of_peptide_type . ")";
	}

	my %adjust_spectral_count_hash;

	###########################################################################
	#Adjust spectral count
	#For each protein, the relative abundance is calculated
	###########################################################################
	foreach my $i ( 0 .. $#pidarray ) {
		$pid_uSPC = $pidarray[$i];

		my $relative_abundance = 0;
		$relative_abundance += $uSPC_hash{$pid_uSPC};

		my %peptide_num_ofthisprotein_hash;

		my @tmparray = split( /\s+/, $protein_pepidlist_hash{$pid_uSPC} );
		foreach my $element (@tmparray) {
			my $peptide_id = substr( $element, 0, 8 );
			my @elementarray = split( /\-/, $element );
			$peptide_num_ofthisprotein_hash{$peptide_id} = $elementarray[2];
		}

		foreach
		  my $pid_group ( sort keys %protein_group_as_key_peptide_list_hash )
		{

			if ( index( $pid_group, $pid_uSPC ) >= 0 ) {

				my $peptide_num_sum = 0;
				@tmparray =
				  split( /\s+/,
					$protein_group_as_key_peptide_list_hash{$pid_group} );
				foreach my $element (@tmparray) {
					if ( exists $peptide_num_ofthisprotein_hash{$element} ) {
						$peptide_num_sum +=
						  $peptide_num_ofthisprotein_hash{$element};
					}
				}

				my $total_uSPC = 0;
				@tmparray = split( /\s+/, $pid_group );
				foreach my $element (@tmparray) {
					$total_uSPC += $uSPC_hash{$element};
				}

				my $share_uSPC = 0;
				if ( $total_uSPC != 0 ) {
					$share_uSPC = $peptide_num_sum *
					  ( $uSPC_hash{$pid_uSPC} / $total_uSPC );
				}
				$relative_abundance += $share_uSPC;
			}
		}

		if ( $relative_abundance != 0 ) {
			$relative_abundance = sprintf("%.10f", $relative_abundance);
			$adjust_spectral_count_hash{$pid_uSPC} = $relative_abundance;
		}
	}

	foreach my $i ( 0 .. $#ppidarray ) {
		my $unique = "";
		my $share  = "";
		my $adjust = "";
		if ( exists $unique_spectral_count_hash{ $ppidarray[$i] } ) {
			$unique = $unique_spectral_count_hash{ $ppidarray[$i] };
		}
		if ( exists $share_spectral_count_hash{ $ppidarray[$i] } ) {
			$share = $share_spectral_count_hash{ $ppidarray[$i] };
		}
		if ( exists $adjust_spectral_count_hash{ $ppidarray[$i] } ) {
			$adjust = $adjust_spectral_count_hash{ $ppidarray[$i] };
		}
    print OUT $adjust . "\n";
	}
	close(OUT);
}

$outafile = "AGIList_totaladjspc.txt";
open( OUTa, ">$outafile" );


foreach my $i (0 .. $#ppidarray ) {
  my $totaladjust = "";
  foreach my $id ( 16 .. 49 ) {
    $fid = "F" . $id;
    $infile = "AGIList_abundance_$fid.txt";
    open( IN, $infile ) or die "open AGIList_abundance_$fid.txt fail ";
    @adjspc = <IN>;
    close(IN);
    $totaladjust += $adjspc[$i];
  }
  print OUTa $ppidarray[$i] . "\t" . $totaladjust . "\n";

}

close(OUTa);




##################################################################################
#Each protein associated with total adjusted spectral counts and protein length
##################################################################################

my $infile = "AGIList_totaladjspc.txt";
open( IN, $infile ) or die "open fail";
@sarray = <IN>;
close(IN);

my $infile = "protein_length.txt";
open(IN, $infile) or die "open fail";
@larray = <IN>;
close(IN);



$outfile = "totaladjspc+length.txt";
open (OUT, ">$outfile");


my $cur0 = "";
my $cur1 = "";
my $cur2 = "";
my $cur3 = "";

foreach my $i ( 0 .. $#sarray ) {

  foreach my $d (0 .. $#larray ) {
    my @parts = split(/\t/, $sarray[$i]);
    $cur1 = $parts[1];
    my @parts1 = split(/\W/, $parts[0]);
    $cur0 = $parts1[0];

    # print $cur0 . "\t" . $cur1 . "\n";
    my @parts2 = split(/\t/, $larray[$d]);
    $cur2 = $parts2[0];
    $cur3 = $parts2[1];
    chomp($cur1);
    chomp($cur2);
    chomp($cur3);
    #print $i;

    if ($cur2 =~ $cur0) {
      print OUT $cur0 . "\t" . $cur3 . "\t" . $cur1 . "\n";
    }

  }
}

close (OUT);





##################################################################################
#Each protein associated with normalized spectral abundance factor
##################################################################################

my $infile = "totaladjspc+length.txt";
open( IN, $infile ) or die "open fail";
@abarray = <IN>;
close(IN);


$outfile = "NSAF.txt";
open (OUT, ">$outfile");

$outffile = "fsdl.txt";
open (OUTf, ">$outffile");

$id = "";
$spc = "";
$length = "";
$sdl = "";
$totalsdl = "";
$NSAF = "";
$fsdl = "";


foreach my $i ( 0 .. $#abarray ) {
  my @info = split(/\s/, $abarray[$i]);
  $id = $info[0];
  $spc = $info[2];
  $length = $info[1];
  $sdl = $spc;
  $sdl /= $length;
  $totalsdl += $sdl;

}



foreach my $i ( 0 .. $#abarray ) {
  my @info = split(/\s/, $abarray[$i]);
  $id = $info[0];
  $spc = $info[2];
  $length = $info[1];
  $sdl = $spc;
  $sdl /= $length;
  $NSAF = $sdl;
  $NSAF /= $totalsdl;
  print OUTf $NSAF . "\n";
  print OUT $id . "\t" . $NSAF . "\n";
}
# print $tNSAF . "\n";

close(OUTf);
close (OUT);



##################################################################################
#Each protein associated with relative protein abundance 
##################################################################################


my $infile = "NSAF.txt";
open( IN, $infile ) or die "open fail";
@nasfarray = <IN>;
close(IN);

my $infile = "fsdl.txt";
open( IN, $infile ) or die "open fail";
@fsdlarray = <IN>;
close(IN);

use List::Util qw[min max];
$max = max @fsdlarray;
$min = min @fsdlarray;
# print $max;
# print $min;


$outfile = "scaling.txt";
open (OUT, ">$outfile");


$scale = "";
$b = 1000;
$a = 1;


foreach my $i ( 0 .. $#nasfarray ) {
  my @info1 = split(/\t/, $nasfarray[$i]);
  $id = $info1[0];
  $NSAF = $info1[1];
  # print $NSAF . "\n";
  $scale = ($b-$a);
  $scale *= ($NSAF-$min);
  $scale /= ($max - $min);
  $scale += 1;
  print OUT $id . "\t" . $scale . "\n";
  


}

close (OUT);


